/*
 * Copyright (c) 2002, Stuart Walsh
 * All rights reserved.
 *
 * This file is licensed under the terms of the GPL license. See the file
 * COPYING for more information.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"

#define FALSE 0
#define TRUE 1

int split_line(char *line, char *command, char *param)
{
    char *tmp;
    int i = 0;
    int found = 0;
    int arg = 0;

    for(tmp = line; *tmp != '\0'; tmp++)
    {
        if(*(tmp+1) == '\0')
        {
            if(arg)
            {
                param[i] = '\0';
                found = TRUE;
            }
        }
        switch(*tmp)
        {
            case '#':
            case ';':
                if(arg)
                    return TRUE;
                else
                    return FALSE;

                break;
            case '\n':
            case '\r':
            case ' ':
            case '\t':
                continue;
            case '=':
                command[i] = '\0';
                arg = TRUE;
                i = 0;
                break;
            default:
                if(arg)
                    param[i] = *tmp;
                else
                    command[i] = *tmp;
                i++;
        }
    }
    if(found)
        return TRUE;
    else
        return FALSE;
}

int do_config(aConf *config, char *command, char *param)
{
    if(!config || !command || !param)
        return FALSE;

    if(strcasecmp(command, "username") == 0)
    {
        config->user = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "password") == 0)
    {
        config->pass = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "hostname") == 0)
    {
        config->host = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "port") == 0)
    {
        config->port = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "cert") == 0)
    {
        config->cert = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "local_ip") == 0)
    {
        config->local = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "remote_ip") == 0)
    {
        config->remote = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "local_interface") == 0)
    {
        config->linterface = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "tunnel_interface") == 0)
    {
        config->tinterface = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "local_endpoint") == 0)
    {
        config->myend = strdup(param);
        return TRUE;
    }
    else if(strcasecmp(command, "remote_endpoint") == 0)
    {
        config->hisend = strdup(param);
        return TRUE;
    }
    return FALSE;
}

int process_conf(struct conf *config, char *name)
{
    char filename[255];
    char *line = calloc(255, sizeof(char));
    char *command, *param;
    FILE *fptr;

    strncpy(filename, name, 255);

    fptr = fopen(filename, "rt");
    if(!fptr)
    {
        perror("ERROR: unable to open config file");
        return FALSE;
    }
    command = calloc(255, sizeof(char));
    if(!command)
    {
        perror("malloc");
        exit(-1);
    }
    param = calloc(255, sizeof(char));
    if(!param)
    {
        perror("malloc");
        exit(-1);
    }

    while(!feof(fptr))
    {
        int foo;

        memset(line, 0, 255);
        line = fgets(line, 255, fptr);
        if(!line)
            continue;

        foo = split_line(line, command, param);
        if(foo)
        {
            do_config(config, command, param);
        }
    }
    free(command);
    free(param);
    fclose(fptr);
    return TRUE;
}
void free_conf(aConf *config)
{
    if(config->user)
        free(config->user);
    if(config->pass)
        free(config->pass);
    if(config->host)
        free(config->host);
    if(config->port)
        free(config->port);
    if(config->cert)
        free(config->cert);
    if(config->remote)
        free(config->remote);
    if(config->local)
        free(config->local);
    if(config->linterface)
        free(config->linterface);
    if(config->tinterface)
        free(config->tinterface);
    if(config->myend)
        free(config->myend);
    if(config->hisend)
        free(config->hisend);
}
